@@ -1,8 +1,8 @@
 /****************************************************************************
 **
-** Copyright (c) 2009-2015 C.B. Barber. All rights reserved.
-** $Id: //main/2015/qhull/src/libqhullcpp/Coordinates.cpp#4 $$Change: 2066 $
-** $DateTime: 2016/01/18 19:29:17 $$Author: bbarber $
+** Copyright (c) 2009-2020 C.B. Barber. All rights reserved.
+** $Id: //main/2019/qhull/src/libqhullcpp/Coordinates.cpp#2 $$Change: 2953 $
+** $DateTime: 2020/05/21 22:05:32 $$Author: bbarber $
 **
 ****************************************************************************/
 
@@ -150,6 +150,7 @@ indexOf(const coordT &t, countT from) const
 countT Coordinates::
 lastIndexOf(const coordT &t, countT from) const
 {
+    countT idx;
     if(from<0){
         from += count();
     }else if(from>=count()){
@@ -159,7 +160,8 @@ lastIndexOf(const coordT &t, countT from) const
         const_iterator i= begin()+from+1;
         while(i-- != constBegin()){
             if(*i==t){
-                return (static_cast<countT>(i-begin())); // WARN64 coordinate index
+                idx= static_cast<countT>(i-begin()); // WARN64 coordinate index
+                return idx;
             }
         }
     }
